
use COMMAND_TYPES;
use db;
use strict;
use warnings;
use diagnostics;
use Time::HiRes;
use controller_send;

#USAGE: [de]elevate("tcp","1.1.1.1","2.2.2.2",1234,123);

# elevate someone, we should be given a 5-tuple to work with here
#     protocol, src_pip, dst_pip, src_port, dst_port
# note: src and dst order does not matter, as long as src_ip matches src_port.
sub elevate {
    my $type  = shift;
    my ($proto, $src_pip, $dst_pip, $src_port, $dst_port) = split(/\s+/, shift);

    # check to make sure both system's are auth'd
    # if not, let's cancel the order (TODO some nice response message?)
    if( 1 and ( !(checkAuth($src_pip)) || !(checkAuth($dst_pip))) ) {  print "A client failed to authenticate! Returning...\n"; return 1; }
    #`else { print "Skipping client authentication\n"; }
	

    # get the appropriate EIPs relating to the flow from the database
    my ($src_eip, $dst_eip) = getEIPs($src_pip,$src_port);
    print "Controller: got EIPs " . $src_eip . " " . $dst_eip . "\n";

    print "!!Deelevating PIP:" . $src_pip . " EIP:" . $src_eip . "\n";
    print "!!Deelevating PIP:" . $dst_pip . " EIP:" . $dst_eip . "\n";

    # setup variable to hold return values
    my $retval = "";

    # transmit IPtables_Down to host1
    $retval = send_snmp(IPtables_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port);
	print "Controller: Sent IPtables_Down to " . $src_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 60 (type 30 acked by 1 host)

    # transmit IPtables_Down to host2
    $retval = send_snmp(IPtables_Down . " " . time() . " " . $proto . " " . $dst_pip . " " . $src_pip . " " . $dst_eip . " " . $src_eip . " " . $dst_port . " " . $src_port);
	print "Controller: Sent IPtables_Down to " . $dst_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 65 (type 30 acked by 2 hosts)

    my $time_to_exec = time() + 2; # in theory, gets time down to microseconds then adds 5 seconds to it.

    # transmit Conntrack_Down to host1
    #$retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port . " " . $time_to_exec);
    $retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port . " " . $time_to_exec, 1);
	print "Controller: Sent Conntrack_Down to " . $src_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 90 (type 35 acked by 1 host)

    # transmit Conntrack_Down to host2
    #$retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port . " " . $time_to_exec);
    $retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port . " " . $time_to_exec, 2);
	print "Controller: Sent Conntrack_Down to " . $dst_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 95 (type 35 acked by 2 hosts)
    
    #remove the TCP RST drop rules
    $retval = send_snmp(TCP_Rst_Down . " " . time() . " " . $src_pip);
    $retval = send_snmp(TCP_Rst_Down . " " . time() . " " . $dst_pip);
	print "Controller: Sent TCP_Rst_Down " . $src_pip . " & " . $dst_pip . "\n";
    
    # transmit EIP_Down to host1
    $retval = send_snmp(EIP_Down . " " . time() . " " . $src_pip . " " . $src_eip);
	print "Controller: Sent EIP_Down to " . $src_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 75 (type 25 acked by 1 host)
    
    # transmit EIP_Down to host2
    $retval = send_snmp(EIP_Down . " " . time() . " " . $dst_pip . " " . $dst_eip);
	print "Controller: Sent EIP_Down to " . $dst_pip . "\n";
    ## TODO: if statement to verify success
    ## UPDATE STATE IN DB = 80 (type 25 acked by 2 hosts)

    # confirm de-elevation back to the policy manager
    # TODO: send Confirm_Delevate

    # remove from DB
    reElevate($src_eip);
	print "Controller: Told DB to show IP reElevated: " . $src_pip . "\n";

    return 0;
}


# deelevate someone, we should be given a 5-tuple to work with here
#     protocol, src_pip, dst_pip, src_port, dst_port
# note: src and dst order does not matter, as long as src_ip matches src_port.
sub deelevate {
    my $type  = shift;
    my ($proto, $src_pip, $dst_pip, $src_port, $dst_port) = split(/\s+/, shift);

	print "Inside De-elevate (controller-send.pm)\n";


    # check to make sure both system's are auth'd
    # if not, let's cancel the order (TODO some nice response message?)
    if( 1 and ( !(checkAuth($src_pip)) || !(checkAuth($dst_pip))) ) {  print "A client failed to authenticate! Returning...\n"; return 1; }
    #else { print "Skipping client authentication\n"; }

	print "Client was authenticated\n";

    # get the next open EIP from the database, then compute the next sequential ip
    my $src_eip = get();

    #increment the last element of the array
    #NOTE: this only works assuming we never have to 'wrap-around' IPs.
    #that is, this is naive and will increment 255 to 256.
    my @dst_eip_array = split(/\./, $src_eip);
    $dst_eip_array[-1]++;
    my $dst_eip=join(".",@dst_eip_array);

    # setup variable to hold return values
    my $retval = "";
    

	print "Delevate to DB\n";
    deElevate($src_eip, $dst_eip, $src_pip, $dst_pip, $src_port, $dst_port, $proto, time());


    # transmit EIP_Up to host1
	print "EiP UP to $src_pip\n";
    $retval = send_snmp(EIP_Up . " " . time() . " " . $src_pip . " " . $src_eip);
    ## TODO: if statement to verify success
    ## update state in db = 10 (type 15 acked by 1 host)
    updateStatus($src_eip, 10);

    # transmit EIP_Up to host2
	print "EiP UP to $dst_pip\n";
    $retval = send_snmp(EIP_Up . " " . time() . " " . $dst_pip . " " . $dst_eip);
    ## TODO: if statement to verify success
    ## update state in db = 15 (type 15 acked by 2 hosts)
    updateStatus($src_eip, 15);

    # transmit IPtables_Up to host1
	print "IPTABLES UP to $src_pip\n";
    $retval = send_snmp(IPtables_Up . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_eip . " " . $dst_eip . " " . $src_port . " " . $dst_port);
    ## TODO: if statement to verify success
    ## update state in db = 25 (type 20 acked by 1 host)
    updateStatus($src_eip, 25);

    # transmit IPtables_Up to host2
	print "IPTABLES UP to $dst_pip\n";
    $retval = send_snmp(IPtables_Up . " " . time() . " " . $proto . " " . $dst_pip . " " . $src_pip . " " . $dst_eip . " " . $src_eip . " " . $dst_port . " " . $src_port);
    ## TODO: if statement to verify success
    # update state in db = 30 (type 20 acked by 2 hosts)
    updateStatus($src_eip, 30);

    my $time_to_exec = time() + 2; # in theory, gets time down to microseconds then adds 5 seconds to it.

    # transmit Conntrack_Down to host1
    # NOTE- when deelevating we need to send PIP and PIPs, that's why they are duplicated here.
	print "CONNTRACK DOWN TO $src_pip\n";
    $retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_pip . " " . $dst_pip . " " . $src_port . " " . $dst_port . " " . $time_to_exec, 1);
    ## TODO: if statement to verify success
    ## update state in db = 40 (type 25 acked by 1 host)
    updateStatus($src_eip, 40);

    # transmit Conntrack_Down to host2
    # NOTE- when deelevating we need to send PIP and PIPs, that's why they are duplicated here.
	print "CONNTRACK DOWN TO $dst_pip\n";
    $retval = send_snmp(Conntrack_Down . " " . time() . " " . $proto . " " . $src_pip . " " . $dst_pip . " " . $src_pip . " " . $dst_pip . " " . $src_port . " " . $dst_port . " " . $time_to_exec, 2);
    ## TODO: if statement to verify success
    ## update state in db = 45 (type 25 acked by 2 hosts)
    updateStatus($src_eip, 45);

    # transmit tcp_reset_down to host1 & host2, update the DB along the way
	print "TCP RESET DOWN TO $src_pip\n";
    $retval = send_snmp(TCP_Rst_Down . " " . time() . " " . $src_pip);
    updateStatus($src_eip,46);
	print "TCP RESET DOWN TO $dst_pip\n";
    $retval = send_snmp(TCP_Rst_Down . " " . time() . " " . $dst_pip);
    updateStatus($src_eip,47);


    # confirm de-elevation back to the policy manager
    # TODO: send Confirm_Delevate
    ## update state in db = 50 (de-elevated)
    updateStatus($src_eip, 50);

    return 0;
}

sub addAuthenticatedClient {
	my ($ip, $mac, $time) = @_;
	auth($ip, $mac, $time);
}


# Future Policy Manager Work:
# * figure out how to communicate state back to PM. Does it poll or do we send information as it happens to connected PMs?

1;
