#!/usr/bin/perl

use strict;
use warnings;

use Net::SNMP;

# Sender vars
#my $username = 'controller';
my $username = 'authOnly';
my $password = 'password';
my $DHCP_ip_addr = '10.22.2.2';
my $router_ip_addr = '10.22.2.2';
my $OID_base = '.1.3.6.1.4.1.7128.9999.9999';
my $OID_send;

sub send_snmp {
    my $input = shift;
    my $flag = shift;

    my $hostname = parse_hn($input, $flag);
    return 1 unless defined $hostname;

    # Create SNMP session
    my ($session, $error) = Net::SNMP->session(
                           -hostname      => $hostname,
                           -version       => '3',
                           -username      => $username,  # v3
                           -authpassword  => $password,  # v3
                           -authprotocol  => 'sha',      # v3
 #                          -privpassword  => $password,  # v3
 #                          -privprotocol  => 'aes',      # v3
                        );

    # Check for errors
    if (!defined $session) {
        printf "ERROR: %s.\n", $error;
        return 1;
    }

    my $result = $session->set_request(
                          -varbindlist => [ $OID_send, OCTET_STRING, $input ],
                       );

    unless(defined $result) {
        warn $session->error();
        return 1;
    }

    return 0;
}

sub parse_hn {
    my @pieces = split(' ', shift);
    my $flag = shift;

    if($pieces[0] == EIP_Up or $pieces[0] == EIP_Down) {
        $OID_send = $OID_base . ".3"; # sending to a host
        return $pieces[2];            # return the PIP
    }
    elsif($pieces[0] == IPtables_Up or $pieces[0] ==  IPtables_Down){
        $OID_send = $OID_base . ".3"; # sending to a host
        return $pieces[3];            # return the src PIP
    }
    elsif($pieces[0] == Conntrack_Down) {
        $OID_send = $OID_base . ".3"; # sending to a host
        if($flag == 1){
            return $pieces[3];            # return the src PIP
        }
        elsif($flag == 2 ){
            return $pieces[4];            # return the src PIP
        }
    }
    elsif($pieces[0] == TCP_Rst_Down){
        $OID_send = $OID_base . ".3"; # sending to a host
        return $pieces[2];
    }
    elsif($pieces[0] == DHCP_Auth_Req_DHCP) {
        $OID_send = $OID_base . ".1"; # sending to DHCP server
        return $DHCP_ip_addr;
    }
    elsif($pieces[0] == Forward_Traffic) {
        $OID_send = $OID_base . ".4"; # sending to router
        return $router_ip_addr;
    }
    else {
        warn $pieces[0], " is not a recognized command type!\n";
        return undef;
    }
}

1;
