#!/usr/bin/perl

use strict;
use warnings;

use List::Util qw(max min);

########################################################################
########################################################################
########################################################################

# Does not handle well:
#   looping in IP ID or sequence numbers

# x  = bits per second
#
# x / ( 8 * 1514 ) packets per second
# (2^19 * 1514 / x) seconds per loop for IP IDs (assuming ++ for new one)
#
# ( 2^( n + 3 ) * 1514 / x ) seconds per loop at x bits/sec and n bits for id

########################################################################
########################################################################
########################################################################

my $usage = "udp <source file> <dest file>\n" . "tcp <source file> <ack file>\n";

die $usage
    unless 3 == scalar(@ARGV) and ("udp" eq $ARGV[0] or "tcp" eq $ARGV[0]);

my ($proto, $src_fn, $dst_fn) = @ARGV;

open(my $src_fh, '<', $src_fn)
    or die "Unable to open < $src_fn - $!";

open(my $dst_fh, '<', $dst_fn)
    or die "Unable to open < $dst_fn - $!";

if("udp" eq $proto) {
    parse_udp($src_fh, $dst_fh);
}
elsif("tcp" eq $proto) {
    parse_tcp($src_fh, $dst_fh);
}
else {
    print "Unknown protocol!\n";
}

close($dst_fh);
close($src_fh);


sub parse_udp {
    my ($src_fh, $dst_fh) = @_;

    my ($total_packets, $lost_packets, $new, $dst_line) = (0, 0, 1, undef);

    my ($dst_id, $dst_ts, $dst_size);

    while(my $src_line = <$src_fh>) {
        chomp($src_line);
        $total_packets++;

        die "$src_line is improperly formatted!\n"
            unless($src_line =~ m/^(\d+) (\d+\.\d+) (\d+)$/);
        my ($src_id, $src_ts, $src_size) = ($1, $2, $3);

        if($new) {
            $dst_line = <$dst_fh> if $new;
            chomp($dst_line);
            die "$dst_line is improperly formatted!\n"
                unless($dst_line =~ m/^(\d+) (\d+\.\d+) (\d+)$/);
            last unless defined $dst_line;
            ($dst_id, $dst_ts, $dst_size) = ($1, $2, $3);
            $new = 0;
        }

        if($src_id != $dst_id) {
            $lost_packets++;
            next;
        }

        $new = 1;
    }

    print "\n";
    print "lost packets  : ", $lost_packets, "\n";
    print "total packets : ", $total_packets, "\n";

    return;
}

sub parse_tcp {
    my ($src_fh, $ack_fh) = @_;

    my (%src, %ack) = ((), ());
    my @rtt = ();

    parse_file($src_fh, \%src);
    parse_file($ack_fh, \%ack);

    my $retransmissions = 0;

    foreach my $id (keys %src) {
        if(my $new_rt = scalar @{$src{$id}} - 1) {
#            print "retrans\t", $new_rt, "\t", $id, "\n";
            $retransmissions += $new_rt;
            next;
        }

#        warn "More acks received than packets sent for seq. num.\n"
#            if scalar(@{$ack{$id}}) - 1;

        
        push(@rtt, $id . "\t" . abs(min(@{$ack{$id}}) - max(@{$src{$id}})))
            if defined $ack{$id};
    }

    print "retransmissions : ", $retransmissions, "\n";
    print "total packets   : ", scalar keys %src, "\n";
    print "\n";
    
    foreach my $val (@rtt) {
        print $val, "\n";
    }

    return;
}

sub parse_file {
    my ($fh, $h_ref) = @_;
    
    my $num = 0;

    while(my $line = <$fh>) {
        chomp($line);
        die "$line is improperly formatted!\n"
            unless($line =~ m/^(\d+) (\d+\.\d+)/);
        my ($id, $ts) = ($1, $2);

        push(@{$h_ref->{$id}}, $ts);
        $num++;
    }

    return $num;
}
