#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pcap.h>
#include <signal.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <inttypes.h>

#define SRC_IP  1
#define PCAP_FN 2


pcap_t             * handle = NULL;
struct sockaddr_in   dest;
int                  s;
int                  sender_ip;

void handle_packets(u_char *args, const struct pcap_pkthdr *header, const u_char *packet);

int main(int argc, char * argv[]) {
    if(3 != argc) {
        fprintf(stderr, "usage: <src ip> <pcap dump file>\n");
        return -1;
    }

    int rtn;
    char errbuff[PCAP_ERRBUF_SIZE];
    pcap_t * handle;

    sender_ip = inet_addr(argv[SRC_IP]);

    handle = pcap_open_offline(argv[PCAP_FN], errbuff);

    if(NULL == handle) {
        fprintf(stderr, "Unable to create offline packet capture on file \"%s\" - %s\n", argv[PCAP_FN], errbuff);
    }
    rtn = pcap_dispatch(handle, 0, handle_packets, NULL);
    if(0 > rtn) {
        fprintf(stderr, "Error for \"%s\" - %s\n", argv[PCAP_FN], pcap_geterr(handle));
    }

    pcap_close(handle);

    return 0;
}

void handle_packets(u_char *args, const struct pcap_pkthdr *header, const u_char *packet) {
    char * src_ip, * dst_ip;
    struct in_addr temp;

    struct iphdr * i_hdr = NULL;
    uint32_t payload_size;

    bpf_u_int32 caplen = header->caplen;

    // advance past the ethernet header
    packet += sizeof(struct ether_header);
    caplen -= sizeof(struct ether_header);

    // advance past the ip header
    i_hdr   = (struct iphdr*) packet;
    packet += 4*i_hdr->ihl;
    caplen -= 4*i_hdr->ihl;
    payload_size = ntohs(i_hdr->tot_len) - 4*i_hdr->ihl;

    temp.s_addr = i_hdr->saddr;
    src_ip = strdup(inet_ntoa(temp));

    temp.s_addr = i_hdr->daddr;
    dst_ip = strdup(inet_ntoa(temp));

    // TCP
    if(6 == i_hdr->protocol) {
        struct tcphdr * t_hdr = (struct tcphdr *) packet;
        if(t_hdr->syn || t_hdr->fin) return;
        if(sender_ip != i_hdr->saddr) {
            fprintf(stderr, "%u %ld.%06ld\n", ntohl(t_hdr->ack_seq), header->ts.tv_sec, header->ts.tv_usec);
            return;
        }
        payload_size -= sizeof(struct tcphdr);
        printf("%u %ld.%06ld %u\n", ntohl(t_hdr->seq), header->ts.tv_sec, header->ts.tv_usec, payload_size);
    }
    // UDP
    else if(17 == i_hdr->protocol) {
        payload_size -= sizeof(struct udphdr);
        printf("%u %ld.%06ld %u\n", i_hdr->id, header->ts.tv_sec, header->ts.tv_usec, payload_size);
    }

    free(src_ip);
    free(dst_ip);

    return;
}
