#!/usr/bin/perl

use IO::Socket::INET;
use Class::Struct;

use Net::SNMP;

use COMMAND_TYPES;

use Thread;
use Thread::Semaphore;
use threads;
use threads::shared;

# define constants
my ($MAX_AGE, $ELEVATED, $DEELEVATED, $AGE, $MODE) :shared;
($MAX_AGE, $ELEVATED, $DEELEVATED, $AGE, $MODE) = (15, 0, 1, 0, 1);
my $OID : shared;
$OID = '.1.3.6.1.4.1.7128.9999.9999.2';

$| = 1;

# define shared variables
my %packets :shared;
my $semaphore = Thread::Semaphore->new();

# start execution
Thread->new(\&monitor_existing);
incoming();

sub incoming {

    my ($socket, $received_data);

    $socket = new IO::Socket::INET (
        LocalPort => '50003',
        Proto => 'udp',
    ) or die "ERROR in Socket Creation : $!\n";

    while(1) {
        $socket->recv($received_data, 64);

        $semaphore->down();

        print $received_data, "\n";
        
        warn "$received_data has been seen before!\n"
            if defined $packets{$received_data};

        my $temp : shared;
        $temp = time() . " " . $ELEVATED;
        $packets{$received_data} = $temp;

        $semaphore->up();
    }
    $socket->close();
}

sub monitor_existing {
    while(1) {

        $semaphore->down();

        foreach my $tuple (keys %packets) {

            my @struct = split(/\s+/, $packets{$tuple});
            if(time() - $struct[$AGE] > $MAX_AGE) {
                # de-elevate
                print time(), "\t", $struct[$AGE], "\t", time() - $struct[$AGE], "\n";
                if($ELEVATED == $struct[$MODE]) {
                    print "Attempting to de-elevate $tuple\n";
                    send_snmp(De_Elevate . " " . time() . " " . $tuple);
                    my $temp : shared;
                    $temp = time() . " " . $DEELEVATED;
                    $packets{$tuple} = $temp;
                }
                # elevate
                else {
                    print "Attempting to elevate $tuple\n";
                    send_snmp(Re_Elevate . " " . time() . " " . $tuple);
                    my $temp : shared;
                    $temp = time() . " " . $ELEVATED;
                    $packets{$tuple} = $temp;
                }
            }
        }

        $semaphore->up();

        sleep(3);
    }
}

sub send_snmp {
    
    my ($hostname, $username, $password) = ('10.22.2.3', 'controller', 'password');
    
    my $input = shift;
    
    my ($session, $error) = Net::SNMP->session(
                           -hostname      => $hostname,
                           -version       => '3',
                           -username      => $username,  # v3
                           -authpassword  => $password,  # v3
                           -authprotocol  => 'sha',      # v3
                           -privpassword  => $password,  # v3
                           -privprotocol  => 'aes',      # v3
                        );

    if (!defined $session) {
        printf "ERROR: %s.\n", $error;
        return 1;
    }

    my $result = $session->set_request(
                              -varbindlist => [ $OID, OCTET_STRING, $input ],
                           );

    unless(defined $result) {
        print $session->error(), "\n";
        return 1;
    }

    $session->close();
    
    return 0;
}
