#!/usr/bin/perl

use strict;
use warnings;

use Switch;

use COMMAND_TYPES;
use controller_act;
use controller_send;

use Net::SNMP;
use NetSNMP::agent (':all');
use NetSNMP::ASN (':all');

use Time::HiRes qw(time);

$| = 1;

# Agent vars
my $agent;
my $agent_name = "controller-agent";
my $regat = '.1.3.6.1.4.1.7128.9999.9999.2';

# Create SNMP Agent
$agent = new NetSNMP::agent(
                        # makes the agent read a $agent_name.conf file
                        'Name' => $agent_name,
                        'AgentX' => 1
                        );

$agent->register($agent_name, $regat, \&handler);

$agent->main_loop();

warn "Controller agent unexpectedly ended!\n";

exit(0);

sub handler {
    print "In handler\n";
    my ($handler, $registration_info, $request_info, $requests) = @_;
    my $request;

    for($request = $requests; $request; $request = $request->next()) {
        next unless ($request_info->getMode() == MODE_SET_ACTION); # only act in  this mode
my $start = time;

        my $value = $request->getValue();                  # get command

        $value =~ m/^STRING:\s+"(.+)"/s;                   # remove ASN data type
        $value = $1;
        unless($value =~ m/^(\S+)\s+(\S+)\s+(.+)$/s) {     # match type, timestamp, remainder
            warn "error: $value is not properly formatted!\n";
            $request->setError($request_info, SNMP_ERR_BADVALUE);
            next;
        }
        my ($type, $timestamp, $remainder) = ($1, $2, $3); # retrieve type, timestamp, and remainder

        #if(abs($timestamp - time()) > 3) {
            #$request->setError($request_info, SNMP_ERR_NOACCESS);
            #next;
        #}

    print "Going to switch statement\n";

        switch ($type) {
            case DHCP_Auth_Req_Controller {
		print "Attempting to authenticate\n";
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if dhcp_auth_request_controller($type, $remainder);
            }
            case De_Elevate {
        print "De-elevate! with remainder = $remainder\n";
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if deelevate($type, $remainder);
            }
            case Re_Elevate {
	print "Elevate! with remainder = $remainder\n";
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if elevate($type, $remainder);
            }
            case Connection_Status {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if connection_status_request_controller($type, $remainder);
            }
            else {
                warn "error: $type is not a known command type!\n";
                $request->setError($request_info, SNMP_ERR_BADVALUE);
                next;
            }
        }
my $end = time;
print $end - $start, "\n";
    }
}

sub dhcp_auth_request_controller {
    return 1 unless 2 == scalar(@_);

    my $type    = shift;
    my $command = shift;

    my $new_command = DHCP_Auth_Req_DHCP . " " . time() . " " . $command;

    print "Sending DHCP AUTH REQ $new_command\n";
    
    my $return = send_snmp($new_command);
    if($return == 0){
        #auth passed, add the rule at the router
        my ($ip, $mac) = split(' ', $command);
        my $second_command = Forward_Traffic . " " . time() . " " . $ip;
        $return = send_snmp($second_command);
	if($return == 0){
		addAuthenticatedClient($ip,$mac,time());
	}
	return $return;
    }
    return $return;
}

sub connection_status_request_controller {
    return 1 unless 2 == scalar(@_);

    my $type    = shift;
    my $command = shift;

    my $new_command = Connection_Status . " " . time() . " " . $command;

print "Sending $new_command\n";

    return send_snmp($new_command);
}
