#!/usr/bin/perl

use strict;
use warnings;

use Switch;

use COMMAND_TYPES;
use dhcp_auth_hook;

use NetSNMP::agent (':all');
use NetSNMP::ASN (':all');

my $agent;
my $agent_name = "dhcp-agent";
my $regat = '.1.3.6.1.4.1.7128.9999.9999.1';

sub handler {
    my ($handler, $registration_info, $request_info, $requests) = @_;
    my $request;

    for($request = $requests; $request; $request = $request->next()) {
        next unless ($request_info->getMode() == MODE_SET_ACTION); # only act in  this mode

        my $value = $request->getValue();                  # get command

        $value =~ m/^STRING:\s+"(.+)"/s;                   # remove ASN data type
        $value = $1;
        unless($value =~ m/^(\S+)\s+(\S+)\s+(.+)$/s) {     # match type, timestamp, remainder
            warn "error: $value is not properly formatted!\n";
            $request->setError($request_info, SNMP_ERR_BADVALUE);
            next;
        }
        my ($type, $timestamp, $remainder) = ($1, $2, $3); # retrieve type, timestamp, and remainder

        if(abs($timestamp - time()) > 3) {
            $request->setError($request_info, SNMP_ERR_NOACCESS);
            next;
        }

        switch ($type) {
            case DHCP_Auth_Req_DHCP {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(dhcp_auth_hook($type, $remainder));
            }
            else {
                warn "error: $type is not a known command type!\n";
                $request->setError($request_info, SNMP_ERR_BADVALUE);
                next;
            }
        }
    }
}

$agent = new NetSNMP::agent(
                        # makes the agent read a $agent_name.conf file
                        'Name' => $agent_name,
                        'AgentX' => 1
                        );

$agent->register($agent_name, $regat, \&handler);

$agent->main_loop();

warn "DHCP agent unexpectedly ended!\n";

exit(0);
