#!/usr/bin/perl

use strict;
use warnings;

use Switch;

use COMMAND_TYPES;
use eip_up_hook;
use eip_down_hook;
use iptables_up;
use iptables_down;
use conntrack_down;
use tcp_rst_down;

use NetSNMP::agent (':all');
use NetSNMP::ASN (':all');

my $agent;
my $agent_name = "host-agent";
my $regat = '.1.3.6.1.4.1.7128.9999.9999.3';

sub handler {
    my ($handler, $registration_info, $request_info, $requests) = @_;
    my $request;

    for($request = $requests; $request; $request = $request->next()) {
        next unless ($request_info->getMode() == MODE_SET_ACTION); # only act in  this mode

        my $value = $request->getValue();                  # get command
        $value =~ m/^STRING:\s+"(.+)"/s;                   # remove ASN data type
        $value = $1;
        unless($value =~ m/^(\S+)\s+(\S+)\s+(.+)$/s) {     # match type, timestamp, remainder
            warn "error: $value is not properly formatted!\n";
            $request->setError($request_info, SNMP_ERR_BADVALUE);
            next;
        }
        my ($type, $timestamp, $remainder) = ($1, $2, $3); # retrieve type, timestamp, and remainder

#if(abs($timestamp - time()) > 3) {
        if(0) {
            $request->setError($request_info, SNMP_ERR_NOACCESS);
            next;
        }

	print(time() . " Received SNMP type " . $type . " with message: " . $remainder . "\n");

        switch ($type) {
            case EIP_Up {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(eip_up($type, $remainder));
            }
            case IPtables_Up {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(iptables_up($type, $remainder));
            }
            case Conntrack_Down {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(conntrack_down($type, $remainder));
            }
            case IPtables_Down {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(iptables_down($type, $remainder));
            }
            case EIP_Down {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(eip_down($type, $remainder));
            }
            case TCP_Rst_Down {
                $request->setError($request_info, SNMP_ERR_COMMITFAILED)
                    if(tcp_rst_down($type, $remainder));
            }
            case Connection_Status {
                print "TODO: add connection status hook!\n";
            }
            else {
                warn "error: $type is not a known command type!\n";
                $request->setError($request_info, SNMP_ERR_BADVALUE);
                next;
            }
        }
    }
}

$agent = new NetSNMP::agent(
                        # makes the agent read a $agent_name.conf file
                        'Name' => $agent_name,
                        'AgentX' => 1
                        );

$agent->register($agent_name, $regat, \&handler);

$agent->main_loop();

warn "Host agent unexpectedly ended!\n";

exit(0);
