#!/usr/bin/perl

use strict;

my $DEBUG = 1;

my $USAGE = "./rate_limit.pl [-p proto] [-m mark] [-sp srcPort] [-dp dstPort] [-sip srcIP] [-dip dstIP]\nNOTE: at most one port and one IP may be specified.";
my $USAGE = "./rate_limit.pl [-p proto] [-m mark] [-sp srcPort] [-dp dstPort] [-r rate (kbit)]\nNOTE: at most one port may be specified.";
my %argHash;
initArgs(\%argHash);
parseArgs(\%argHash);
enforceArgs(\%argHash);
printArgs(\%argHash);

addTC(\%argHash);
addIpTables(\%argHash);

sub addTC {
	my $argHash = shift;
	my $ret = `tc qdisc add dev eth0 root handle 1:0 htb default 99`;
	if($DEBUG){
		print "Executing: tc class add dev eth0 parent 1:0 classid 1:10 htb rate $argHash->{'-r'}kbit prio 0\n";
	}
	$ret = `tc class add dev eth0 parent 1:0 classid 1:10 htb rate $argHash->{'-r'}kbit prio 1`;
	$ret = `tc class add dev eth0 parent 1:0 classid 1:99 htb rate 1000mbps ceil 1000mbps prio 0`;
	$ret = `tc filter add dev eth0 parent 1:0 prio 0 protocol ip handle 10 fw flowid 1:10`;
}

sub addIpTables {
	my $argHash = shift;
	my $mark = $argHash->{'-m'};
	my $sp = $argHash->{'-sp'};
	my $dp = $argHash->{'-dp'};
	my $sip = $argHash->{'-sip'};
	my $dip = $argHash->{'-sip'};

	if($sp > 0) {
		if($DEBUG){
			print "Executing: iptables -A OUTPUT -t mangle -p $argHash->{'-p'} --sport $argHash->{'-sp'} -j MARK --set-mark $mark\n";
		}
		my $ret = `iptables -A OUTPUT -t mangle -p $argHash->{'-p'} --sport $argHash->{'-sp'} -j MARK --set-mark $mark`;
	} else{
		if($DEBUG){
			print "Executing: iptables -A OUTPUT -t mangle -p $argHash->{'-p'} --dport $argHash->{'-dp'} -j MARK --set-mark $mark\n";
		}
		my $ret = `iptables -A OUTPUT -t mangle -p $argHash->{'-p'} --dport $argHash->{'-dp'} -j MARK --set-mark $mark`;
	}

	# Some of this is dependent on either having a src or dst port and sip or dip. we really may have more information. thoughts?
	#if($sp > 0){
	#	if($sip != "-1"){
	#	} else{
	#	}
	#} else{
	#	if($sip != "-1"){
	#	} else{
	#	}
	#}
}


sub initArgs {
	my $argHash = shift;

	$argHash->{'-p'} = -1;
	$argHash->{'-m'} = 10;
	$argHash->{'-r'} = -1;
	$argHash->{'-sp'} = -1;
	$argHash->{'-dp'} = -1;
	$argHash->{'-sip'} = -1;
	$argHash->{'-dip'} = -1;
    $argHash->{'-r'} = -1;
}

sub parseArgs {
	my $argHash = shift;

	my $numReqArgs = 8;

	my $len = scalar(@ARGV);

	if($len!= $numReqArgs){
		print "Number of arguments is incorrect\n";
		print $len . " != " . $numReqArgs . "\n";
		print @ARGV;
		print $USAGE;
		exit(1);
	}

	my $argNum = 0;

	while($argNum < $numReqArgs){
		if(exists($argHash{$ARGV[$argNum]})){
#print "Setting key " . $ARGV[$argNum] . " = " . $ARGV[$argNum+1] ."\n";
			$argHash->{$ARGV[$argNum]} = $ARGV[$argNum+1];
			$argNum += 2;
		} else{
			print "Unregcognized parameter " . $ARGV[$argNum] . "\n";
			print $USAGE;
			exit(1);
		}
	}
}

sub enforceArgs {
	my $argHash = shift;

	if(	$argHash->{'-p'} == -1
		or $argHash->{'-m'} == -1
		or $argHash->{'-r'} == -1
		or ($argHash->{'-sp'} == -1 and $argHash->{'-dp'} == -1)){
			#or ($argHash->{'-sip'} == -1 and $argHash->{'-dip'} == -1)){
			print "Not all required fields are set\n";
			print $USAGE;
			exit(1);
		}

	my $lower_limit=1;
	my $upper_limit=1000000;

	if($argHash{'-r'} > $upper_limit  or $argHash{'-r'} < $lower_limit){
		print "Rate limit out of bounds\n";
		print $USAGE;
		exit(1);
	}

}
sub printArgs{
	my $argHash = shift;
	print "Arguments and values:\n";
	for my $key (keys %argHash){
		print "$key = " . $argHash->{$key} . "\n";
	}
}

	




















