#!/usr/bin/python

import os
import sys
import time

EXECDISABLED = False

class host:
	router = "localhost"
	controller = "10.22.2.3"
	policyman = "10.22.2.4"
	clientnet = "10.22.0."

class myv:
	mytime = str(time.time())
	newlogpath       = "~/panopt_logs/" + mytime
	newlogpath_full  = "/home/crtaylor/panopt_logs/" + mytime
	curlogpath       = "~/panopt_logs/cur"
	curlogpath_full  = "/home/crtaylor/panopt_logs/cur"
	codeall          = "~/cs558_fresh/code/all"
	codeall_full     = "/home/crtaylor/cs558_fresh/code/all"

class mycolor:
	HEAD = "\033[95m"
	B = "\033[94m"
	G = "\033[92m"
	WARN = '\033[93m'
	FAIL = '\033[91m'
	ENDC = '\033[0m'

def myexec(host,cmd):
	if host == "localhost":
		torun = cmd
	else:
		torun = "ssh " + host + " '" + cmd + " '"
	myprint(mycolor.G,"@" + host + "\t" + torun)
	if not EXECDISABLED: os.system(torun)

def myprint(color,message):
	print color + message + mycolor.ENDC


# Main Code, first check we are root and have client count parameter
if os.geteuid() != 1000:
	sys.exit("\n*Need to run as crtaylor user for SSH keys to work!\n")
if len(sys.argv) != 2:
	sys.exit("\nIncorrect parameter count, Usage: ./panopt client_host_count\n")

# number of clients 
clientCount = int(sys.argv[1])


## CLEAN UP ##
# Kill Perls
myprint(mycolor.B,"Killing all open perl scripts")
myexec(host.router,"sudo killall perl")
myexec(host.router,"sudo killall monitor")
myexec(host.controller,"sudo killall perl") 
myexec(host.policyman,"sudo killall perl")

# Transfer Old Log Files--
	# We copy from ~/panopt_logs/cur/*
	# If it doesn't exist, it will simply fail and output the error then we move on
myprint(mycolor.B,"Pulling log files back to the router")
myexec(host.router,"scp " + host.controller + ":" + myv.curlogpath + "/* " + myv.curlogpath)
myexec(host.router,"scp " + host.policyman + ":" + myv.curlogpath + "/* " + myv.curlogpath)
for addr in range(0,clientCount):
	myexec(host.router,"scp " + host.clientnet + str((addr*4)+100) + ":~/client_boot_log.txt " + myv.curlogpath + "/" + str((addr*4)+100) + "_client_boot_log.txt")
	myexec(host.router,"scp " + host.clientnet + str((addr*4)+100) + ":~/client_snmp_host_log.txt " + myv.curlogpath + "/" + str((addr*4)+100) + "_client_snmp_host_log.txt")

## REFRESH SYSTEM ##
myprint(mycolor.B,"Restoring an empty database for the contoller to use")
myexec(host.controller,"cp " + myv.codeall + "/database_clean.db " + myv.codeall + "/database.db")

myprint(mycolor.B,"Restarting SNMP Service")
myexec(host.router,"sudo service snmpd restart")
myexec(host.controller,"sudo service snmpd restart")
myexec(host.policyman,"sudo service snmpd restart")

# set up routing
myprint(mycolor.B,"Enabling IPv4 Forwarding")
myexec(host.router,"sudo sysctl net/ipv4/ip_forward=1")

myprint(mycolor.B,"Configuring Routes (seeing messages about it existing are normal here)")
myexec(host.router,"sudo route add -host 10.22.0.1 dev eth0")
myexec(host.router,"sudo route add -net 10.22.0.0 netmask 255.255.0.0 gw 10.22.2.2 metric 10 dev eth0")

# iptables 
myprint(mycolor.B,"Flushing Router IPTables")
myexec(host.router,"sudo iptables -F")
myexec(host.router,"sudo iptables -t nat -F")

myprint(mycolor.B,"Configuring IPTables")
myexec(host.router,"sudo iptables -A OUTPUT -p icmp --icmp-type 5 -j DROP")
myexec(host.router,"sudo iptables -A FORWARD -p udp -m udp --dport 161 -j ACCEPT")
myexec(host.router,"sudo iptables -A FORWARD -p udp -m udp --sport 161 -j ACCEPT")
myexec(host.router,"sudo iptables -P FORWARD DROP")
myexec(host.router,"sudo iptables -A FORWARD --src 10.22.2.0/24  -j ACCEPT")
myexec(host.router,"sudo iptables -A FORWARD --dst 10.22.0.1 -j ACCEPT")

# create all the log folders
myprint(mycolor.B,"Creating new logging folders")
myexec(host.router,"mkdir -p " + myv.newlogpath)
myexec(host.router,"ln -sfn " + myv.newlogpath_full + " " + myv.curlogpath_full)
myexec(host.controller,"mkdir -p " + myv.newlogpath)
myexec(host.controller,"ln -sfn " + myv.newlogpath_full + " " + myv.curlogpath_full)
myexec(host.policyman,"mkdir -p " + myv.newlogpath)
myexec(host.policyman,"ln -sfn " + myv.newlogpath_full + " " + myv.curlogpath_full)

# Don't need because clients have log files in a particular place anyway
#myprint(mycolor.B,"Creating new logging folders (on clients)")
#for addr in range(0,clientCount):
#	myexec(host.clientnet + str((addr*4)+100),"mkdir -p " + myv.newlogpath)
#	myexec(host.clientnet + str((addr*4)+100),"ln -sfn " + myv.newlogpath_full + " " + myv.curlogpath_full)


## START AGENTS ##
myprint(mycolor.B,"Starting DHCP Agent, Router Agent, and Monitor")
myprint(mycolor.G,"@localhost\t(chdir " + myv.codeall_full + ")")
os.chdir(myv.codeall_full)
myexec(host.router,"sudo perl snmp-router-agent.pl 2> " + myv.curlogpath + "/snmp-router-agent.log 1>&2 &")
myexec(host.router,"sudo perl snmp-dhcp-agent.pl 2> " + myv.curlogpath + "/snmp-dhcp-agent.log 1>&2 &")
myexec(host.router,"make")
myexec(host.router,"sudo ./monitor 2> " + myv.curlogpath + "/monitor.log 1>&2 &")

myprint(mycolor.B,"Connecting to Controller and starting agent")
myexec(host.controller,"cd " + myv.codeall_full + "; sudo perl snmp-controller-agent.pl 2> " + myv.curlogpath + "/snmp-controller-agent.log 1>&2 &")
myprint(mycolor.B,"Connecting to Policy Manager and starting agent")
myexec(host.policyman,"cd " + myv.codeall_full + "; sudo perl policy_manager.pl > " + myv.curlogpath + "/policy_manager.log &")

#print mycolor.FAIL + " - Start Controller Agents..." + mycolor.ENDC
#print mycolor.FAIL + " - Start Policy Manager Agents..." + mycolor.ENDC

# Reboot our client machines
myprint(mycolor.B,"Rebooting the client machines")
for addr in range(0,clientCount):
	myexec(host.clientnet + str((addr*4)+100), "sudo shutdown -r now")




